# SPDX-License-Identifier: Apache-2.0

from functools import wraps

from itsdangerous.url_safe import URLSafeTimedSerializer
from flask import g, request
from sqlalchemy import select
from sqlalchemy.orm import Session

from foxfiles.errors import Errors
from foxfiles.settings import SECRET_KEY
from foxfiles.db import User, engine


def require_user(f):
    @wraps(f)
    def inner(*args, **kwargs):
        if "user" not in g:
            return Errors.MISSING_TOKEN

        return f(*args, **kwargs)

    return inner


def maybe_token(f):
    @wraps(f)
    def inner(*args, **kwargs):
        token = request.headers.get("Authorization")
        if not token:
            return f(*args, **kwargs)

        _, id = URLSafeTimedSerializer(SECRET_KEY).loads_unsafe(token)
        try:
            id_int = int(id)
        except:
            return Errors.INVALID_TOKEN

        with Session(engine) as session:
            u: User = session.scalar(select(User).where(User.id == id_int))
            if u is None:
                return Errors.INVALID_TOKEN

            if not u.verify_token(token):
                return Errors.INVALID_TOKEN
            g.user = u

        return f(*args, **kwargs)

    return inner
